// Crimson Fields -- a game of tactical warfare
// Copyright (C) 2000-2002 Jens Granseuer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

/////////////////////////////////////////////////////////////////////////
// lset.h - level set handling
// A level set contains the definitions of terrain and unit types as
// well as the corresponding graphics and sound data.
//
// Changes:
/////////////////////////////////////////////////////////////////////////

#ifndef _INCLUDE_LSET_H
#define _INCLUDE_LSET_H

#include <string>
using namespace std;

#include "map.h"
#include "unit.h"
#include "sound.h"
#include "surface.h"

#define FID_SET       MakeID('M','S','E','T')  // mission set

// class definitions

class LevelSet {
public:
  LevelSet( void );
  ~LevelSet( void );

  int Load( const char *setname );

  const TerrainType *GetTerrainInfo( unsigned short ttid ) const;
  const UnitType *GetUnitInfo( unsigned short utid ) const;
  SoundEffect *GetSound( unsigned short sfxid ) const;
  Surface *GetTiles( void ) { return &tiles; }
  const string &GetName( void ) const { return name; }

private:
  int LoadTerrainTypes( SDL_RWops *file );
  int LoadUnitTypes( SDL_RWops *file );
  int LoadGfx( SDL_RWops *file );
  int LoadSfx( SDL_RWops *file );

  unsigned short num_tt;
  unsigned short num_ut;
  unsigned short num_sfx;

  TerrainType *tt;
  UnitType *ut;
  SoundEffect **sfx;
  Surface tiles;

  string name;
};

#endif	/* _INCLUDE_LSET_H */

